==> bitcoin-0.14.1.tar.gz: bitcoin-0.14.1/src/test/undo_tests.cpp <==
// Copyright (c) 2017 Amaury SÉCHET
// Distributed under the MIT software license, see the accompanying
// file COPYING or http://www.opensource.org/licenses/mit-license.php.

#include "undo.h"
#include "chainparams.h"
#include "consensus/validation.h"
#include "validation.h"

#include "test/test_bitcoin.h"

#include <boost/test/unit_test.hpp>

BOOST_FIXTURE_TEST_SUITE(undo_tests, BasicTestingSetup)

static void UpdateUTXOSet(const CBlock &block, CCoinsViewCache &view,
                          CBlockUndo &blockundo,
                          const CChainParams &chainparams, uint32_t nHeight) {
    CValidationState state;

    auto &coinbaseTx = *block.vtx[0];
    UpdateCoins(coinbaseTx, view, nHeight);

    for (size_t i = 1; i < block.vtx.size(); i++) {
        auto &tx = *block.vtx[1];

        blockundo.vtxundo.push_back(CTxUndo());
        UpdateCoins(tx, view, blockundo.vtxundo.back(), nHeight);
    }

    view.SetBestBlock(block.GetHash());
}

static void UndoBlock(const CBlock &block, CCoinsViewCache &view,
                      const CBlockUndo &blockundo,
                      const CChainParams &chainparams, uint32_t nHeight) {
    CValidationState state;

    CBlockIndex pindex;
    pindex.nHeight = nHeight;

    ApplyBlockUndo(block, state, &pindex, view, blockundo);
}

static bool HashSpendableCoin(const CCoinsViewCache &view, uint256 txid) {
    const CCoins *coins = view.AccessCoins(txid);
    return coins != nullptr && !coins->IsPruned();
}

BOOST_AUTO_TEST_CASE(connect_utxo_extblock) {
    SelectParams(CBaseChainParams::MAIN);
    const CChainParams &chainparams = Params();

    CBlock block;
    CMutableTransaction tx;

    CCoinsView coinsDummy;
    CCoinsViewCache view(&coinsDummy);

    block.hashPrevBlock = GetRandHash();
    view.SetBestBlock(block.hashPrevBlock);

    // Create a block with coinbase and resolution transaction.
    tx.vin.resize(1);
    tx.vin[0].scriptSig.resize(10);
    tx.vout.resize(1);
    tx.vout[0].nValue = 42;
    auto coinbaseTx = CTransaction(tx);

    block.vtx.resize(2);
    block.vtx[0] = MakeTransactionRef(tx);

    tx.vout[0].scriptPubKey = CScript() << OP_TRUE;
    tx.vin[0].prevout.hash = GetRandHash();
    tx.vin[0].prevout.n = 0;
    tx.vin[0].nSequence = CTxIn::SEQUENCE_FINAL;
    tx.vin[0].scriptSig.resize(0);
    tx.nVersion = 2;

    auto prevTx0 = CTransaction(tx);
    view.ModifyNewCoins(prevTx0.GetId(), prevTx0.IsCoinBase())
        ->FromTx(prevTx0, 100);

    tx.vin[0].prevout.hash = prevTx0.GetId();
    auto tx0 = CTransaction(tx);
    block.vtx[1] = MakeTransactionRef(tx0);

    // Now update the UTXO set.
    CBlockUndo blockundo;
    UpdateUTXOSet(block, view, blockundo, chainparams, 123456);

    BOOST_CHECK(view.GetBestBlock() == block.GetHash());
    BOOST_CHECK(HashSpendableCoin(view, coinbaseTx.GetId()));
    BOOST_CHECK(HashSpendableCoin(view, tx0.GetId()));
    BOOST_CHECK(!HashSpendableCoin(view, prevTx0.GetId()));

    UndoBlock(block, view, blockundo, chainparams, 123456);

    BOOST_CHECK(view.GetBestBlock() == block.hashPrevBlock);
    BOOST_CHECK(!HashSpendableCoin(view, coinbaseTx.GetId()));
    BOOST_CHECK(!HashSpendableCoin(view, tx0.GetId()));
    BOOST_CHECK(HashSpendableCoin(view, prevTx0.GetId()));
}

BOOST_AUTO_TEST_SUITE_END()

==> bitcoin-0.14.4.tar.gz: bitcoin-0.14.4/src/test/undo_tests.cpp <==
// Copyright (c) 2017 Amaury SÉCHET
// Distributed under the MIT software license, see the accompanying
// file COPYING or http://www.opensource.org/licenses/mit-license.php.

#include "undo.h"
#include "chainparams.h"
#include "consensus/validation.h"
#include "validation.h"

#include "test/test_bitcoin.h"

#include <boost/test/unit_test.hpp>

BOOST_FIXTURE_TEST_SUITE(undo_tests, BasicTestingSetup)

static void UpdateUTXOSet(const CBlock &block, CCoinsViewCache &view,
                          CBlockUndo &blockundo,
                          const CChainParams &chainparams, uint32_t nHeight) {
    CValidationState state;

    auto &coinbaseTx = *block.vtx[0];
    UpdateCoins(coinbaseTx, view, nHeight);

    for (size_t i = 1; i < block.vtx.size(); i++) {
        auto &tx = *block.vtx[1];

        blockundo.vtxundo.push_back(CTxUndo());
        UpdateCoins(tx, view, blockundo.vtxundo.back(), nHeight);
    }

    view.SetBestBlock(block.GetHash());
}

static void UndoBlock(const CBlock &block, CCoinsViewCache &view,
                      const CBlockUndo &blockundo,
                      const CChainParams &chainparams, uint32_t nHeight) {
    CValidationState state;

    CBlockIndex pindex;
    pindex.nHeight = nHeight;

    ApplyBlockUndo(block, state, &pindex, view, blockundo);
}

static bool HasSpendableCoin(const CCoinsViewCache &view, const uint256 &txid) {
    return !view.AccessCoin(COutPoint(txid, 0)).IsSpent();
}

BOOST_AUTO_TEST_CASE(connect_utxo_extblock) {
    SelectParams(CBaseChainParams::MAIN);
    const CChainParams &chainparams = Params();

    CBlock block;
    CMutableTransaction tx;

    CCoinsView coinsDummy;
    CCoinsViewCache view(&coinsDummy);

    block.hashPrevBlock = GetRandHash();
    view.SetBestBlock(block.hashPrevBlock);

    // Create a block with coinbase and resolution transaction.
    tx.vin.resize(1);
    tx.vin[0].scriptSig.resize(10);
    tx.vout.resize(1);
    tx.vout[0].nValue = 42;
    auto coinbaseTx = CTransaction(tx);

    block.vtx.resize(2);
    block.vtx[0] = MakeTransactionRef(tx);

    tx.vout[0].scriptPubKey = CScript() << OP_TRUE;
    tx.vin[0].prevout.hash = GetRandHash();
    tx.vin[0].prevout.n = 0;
    tx.vin[0].nSequence = CTxIn::SEQUENCE_FINAL;
    tx.vin[0].scriptSig.resize(0);
    tx.nVersion = 2;

    auto prevTx0 = CTransaction(tx);
    view.ModifyNewCoins(prevTx0.GetId(), prevTx0.IsCoinBase())
        ->FromTx(prevTx0, 100);

    tx.vin[0].prevout.hash = prevTx0.GetId();
    auto tx0 = CTransaction(tx);
    block.vtx[1] = MakeTransactionRef(tx0);

    // Now update the UTXO set.
    CBlockUndo blockundo;
    UpdateUTXOSet(block, view, blockundo, chainparams, 123456);

    BOOST_CHECK(view.GetBestBlock() == block.GetHash());
    BOOST_CHECK(HasSpendableCoin(view, coinbaseTx.GetId()));
    BOOST_CHECK(HasSpendableCoin(view, tx0.GetId()));
    BOOST_CHECK(!HasSpendableCoin(view, prevTx0.GetId()));

    UndoBlock(block, view, blockundo, chainparams, 123456);

    BOOST_CHECK(view.GetBestBlock() == block.hashPrevBlock);
    BOOST_CHECK(!HasSpendableCoin(view, coinbaseTx.GetId()));
    BOOST_CHECK(!HasSpendableCoin(view, tx0.GetId()));
    BOOST_CHECK(HasSpendableCoin(view, prevTx0.GetId()));
}

BOOST_AUTO_TEST_SUITE_END()
